@@ -177,7 +177,7 @@ class TripStopTime:
     stop = attr.ib(default=attr.NOTHING)
     dts_arr = attr.ib(default=attr.NOTHING)
     dts_dep = attr.ib(default=attr.NOTHING)
-    fare = attr.ib(default=0)
+    fare = attr.ib(default=0.0)
 
     def __hash__(self):
         return hash((self.trip, self.stopidx))
@@ -201,7 +201,7 @@ class TripStopTimes:
         self.stop_trip_idx: Dict[Stop, List[TripStopTime]] = defaultdict(list)
 
     def __repr__(self):
-        return f"Trips(n_trips={len(self.set_idx)})"
+        return f"TripStoptimes(n_tripstoptimes={len(self.set_idx)})"
 
     def __getitem__(self, trip_id):
         return self.set_idx[trip_id]
@@ -277,8 +277,11 @@ class Trip:
 
     def add_stop_time(self, stop_time: TripStopTime):
         """Add stop time"""
-        assert stop_time.dts_arr <= stop_time.dts_dep
-        assert not self.stop_times or self.stop_times[-1].dts_dep <= stop_time.dts_arr
+        if np.isfinite(stop_time.dts_arr) and np.isfinite(stop_time.dts_dep):
+            assert stop_time.dts_arr <= stop_time.dts_dep
+            assert (
+                not self.stop_times or self.stop_times[-1].dts_dep <= stop_time.dts_arr
+            )
         self.stop_times.append(stop_time)
 
     def get_stop(self, stop: Stop) -> TripStopTime:
@@ -335,10 +338,7 @@ class Route:
         return same_type_and_id(self, trip)
 
     def __repr__(self):
-        return "Route(id={0.id}, trips={trips})".format(
-            self,
-            trips=len(self.trips),
-        )
+        return "Route(id={0.id}, trips={trips})".format(self, trips=len(self.trips),)
 
     def __getitem__(self, n):
         return self.trips[n]
@@ -367,16 +367,16 @@ class Route:
         """Returns earliest trip after time dts (sec)"""
         stop_idx = self.stop_index(stop)
         trip_stop_times = [trip.stop_times[stop_idx] for trip in self.trips]
-        trip_stop_times = [tst for tst in trip_stop_times if tst.dts_arr >= dts_arr]
-        trip_stop_times = sorted(trip_stop_times, key=attrgetter("dts_arr"))
+        trip_stop_times = [tst for tst in trip_stop_times if tst.dts_dep >= dts_arr]
+        trip_stop_times = sorted(trip_stop_times, key=attrgetter("dts_dep"))
         return trip_stop_times[0].trip if len(trip_stop_times) > 0 else None
 
     def earliest_trip_stop_time(self, dts_arr: int, stop: Stop) -> TripStopTime:
         """Returns earliest trip stop time after time dts (sec)"""
         stop_idx = self.stop_index(stop)
         trip_stop_times = [trip.stop_times[stop_idx] for trip in self.trips]
-        trip_stop_times = [tst for tst in trip_stop_times if tst.dts_arr >= dts_arr]
-        trip_stop_times = sorted(trip_stop_times, key=attrgetter("dts_arr"))
+        trip_stop_times = [tst for tst in trip_stop_times if tst.dts_dep >= dts_arr]
+        trip_stop_times = sorted(trip_stop_times, key=attrgetter("dts_dep"))
         return trip_stop_times[0] if len(trip_stop_times) > 0 else None
 
 
@@ -639,7 +639,7 @@ class Bag:
         return min([self.labels[i].earliest_arrival_time for i in range(len(self))])
 
 
-@dataclass
+@dataclass(frozen=True)
 class Journey:
     """
     Journey from origin to destination specified as Legs
@@ -647,7 +647,7 @@ class Journey:
 
     legs: List[Leg] = field(default_factory=list)
 
-    @property  # TODO: make fare, travel time etc properties?
+    @property
     def criteria(self):
         """Criteria"""
         return [self.travel_time(), self.fare(), self.number_of_trips()]
@@ -672,21 +672,25 @@ class Journey:
         trips = set([l.trip for l in self.legs])
         return len(trips)
 
-    def prepend_leg(self, leg: Leg):
+    def prepend_leg(self, leg: Leg) -> Journey:
         """Add leg to journey"""
-        self.legs.insert(0, leg)
+        legs = self.legs
+        legs.insert(0, leg)
+        jrny = Journey(legs=legs)
+        return jrny
 
-    def remove_transfer_legs(self):
+    def remove_transfer_legs(self) -> Journey:
         """Remove all transfer legs"""
-        self.legs = [
+        legs = [
             leg
             for leg in self.legs
             if (leg.trip is not None) and (leg.from_stop.station != leg.to_stop.station)
         ]
+        jrny = Journey(legs=legs)
+        return jrny
 
-    def is_valid(self):
+    def is_valid(self) -> bool:
         """Is valid journey"""
-        self.remove_transfer_legs()
         for index in range(len(self.legs) - 1):
             if self.legs[index].arr > self.legs[index + 1].dep:
                 return False
@@ -763,8 +767,7 @@ class Journey:
         msg = f"Duration: {sec2str(self.travel_time())}"
         if dep_secs:
             msg += " ({} from request time {})".format(
-                sec2str(self.arr() - dep_secs),
-                sec2str(dep_secs),
+                sec2str(self.arr() - dep_secs), sec2str(dep_secs),
             )
         logger.info(msg)
         logger.info("")
