@@ -2,6 +2,7 @@
 import argparse
 from typing import Dict, List
 from copy import copy
+from time import perf_counter
 
 from loguru import logger
 
@@ -142,6 +143,9 @@ def run_range_mcraptor(
         station_name: [] for station_name, _ in destination_stops.items()
     }
 
+    logger.info("Calculating journeys to all destinations")
+    s = perf_counter()
+
     # Find Pareto-optimal journeys for all possible departure times
     for dep_index, dep_secs in enumerate(potential_dep_secs):
         logger.info(f"Processing {dep_index} / {len(potential_dep_secs)}")
@@ -149,8 +153,11 @@ def run_range_mcraptor(
 
         # Run Round-Based Algorithm
         mcraptor = McRaptorAlgorithm(timetable)
-        bag_round_stop = mcraptor.run(from_stops, dep_secs, rounds)
-        last_round_bag = copy(bag_round_stop[rounds])
+        if dep_index == 0:
+            bag_round_stop, actual_rounds = mcraptor.run(from_stops, dep_secs, rounds)
+        else:
+            bag_round_stop, actual_rounds = mcraptor.run(from_stops, dep_secs, rounds, last_round_bag)
+        last_round_bag = copy(bag_round_stop[actual_rounds])
 
         # Determine the best destination ID, destination is a platform
         for destination_station_name, to_stops in destination_stops.items():
@@ -164,6 +171,8 @@ def run_range_mcraptor(
                 )
                 journeys_to_destinations[destination_station_name].extend(journeys)
 
+    logger.info(f"Journey calculation time: {perf_counter() - s}")
+
     # Keep Pareto-optimal journeys
     for destination_station_name, journeys in journeys_to_destinations.items():
         best_journeys = pareto_set(journeys, keep_equal=True)
